;**************************************************************************
;                      EduKit84 Example #3: Blinking
;**************************************************************************
;
; Task specification
;   Pushbutton dependent blink period. Use the internal timer.
;
; Task Purpose
;   To become familiar with the internal timer.
;
; Task analysis
;   A subroutine which receives information about delay from a general
;   purpose bit will be created. This subroutine contains a loop with a
;   parameter and waits for the internal timer overflow. Main program
;   contents: test of pushbutton, setting of universal bit, turning LED on,
;   calling delay subroutine, turning LED off, calling delay subroutine and
;   jump to beginning of the loop in main program.

; DEFINITIONS =============================================================
        list    P=16F84A, N=0, R=hex ;part, no pages in listing, radix
        __config 0x3FF1              ;configuration word (fuses) setup
w       equ     0
f       equ     1
; Special Function Registers ..............................................
tmr0    equ     0x01
optreg  equ     0x01
status  equ     0x03
porta   equ     0x05
trisa   equ     0x05
portb   equ     0x06
trisb   equ     0x06
intcon  equ     0x0B
; User Registers ..........................................................
cnt     equ     0x0C
flags   equ     0x0D
; Bit Definitions .........................................................
#define LED     porta,4         ;LED on the RA4 pin
#define PB      portb,0         ;pushbutton on the RB0 pin
#define T0IF    intcon,2        ;TMR0 overflow indication
#define RP0     status,5
#define BIT     flags,0         ;general purpose bit

; PROGRAM START ===========================================================
        org     0
        goto    Main

; SUBROUTINES =============================================================
Wait    bcf     T0IF            ;T0IF flag clear
        clrf    tmr0            ;TMR0 and prescaler clear
        movlw   d'10'
        btfsc   BIT
        movlw   d'5'
        movwf   cnt
Wait_A  btfss   T0IF            ;TMR0 overflow test
        goto    Wait_A          ;test TMR0 until overflow
        bcf     T0IF            ;T0IF flag clear
        decfsz  cnt,f
        goto    Wait_A          ;outer loop
        return

; MAIN PROGRAM ============================================================
Main    bsf     RP0             ;RAM bank 1
        movlw   b'11101111'
        movwf   trisa           ;pin RA4 is an output
        movlw   b'11010100'
        movwf   optreg          ;TMR0 and prescaler configuration
        bcf     RP0             ;RAM bank 0
Main_A  bcf     BIT             ;pushbutton off - 1 sec blinking
        btfss   PB
        bsf     BIT             ;pushbutton off - 0.5 sec blinking
        bcf     LED             ;LED on
        call    Wait
        bsf     LED             ;LED off
        call    Wait
        goto    Main_A
; =========================================================================
        end