;**************************************************************************
;                       EduKit84 Example #5: Counter
;**************************************************************************
;
; Task specification
;   Create a counter 0 - 9999, using the LED display which increments its
;  value with 1 second period.
;
; Task purpose
;   To learn multiplexed driving of a display.
;
; Task analysis
;   Using TMR0, create time intervals with period 5 ms. 200 of these
;   intervals completes 1 second. In next 5 ms period, turn the next
;   seven-segment display. To avoid complicated conversions of values, use
;   less efficient way: keep the value in four registers and ensure carry
;   from digit 9 to 10 to upper digit. Thus, values on individual
;   seven-segment displays will correspond directly to appropriate
;   registers.

; DEFINITIONS =============================================================
        list    P=16F84A, N=0, R=hex ;part, no pages in listing, radix
        __config 0x3FF1              ;configuration word (fuses) setup
w       equ     0
f       equ     1
; Special Function Registers ..............................................
indf    equ     0x00
tmr0    equ     0x01
optreg  equ     0x01
pcl     equ     0x02
status  equ     0x03
fsr     equ     0x04
porta   equ     0x05
trisa   equ     0x05
portb   equ     0x06
trisb   equ     0x06
intcon  equ     0x0B
; User Registers ..........................................................
c_disp  equ     0x0C            ;multiplexer counter
disp1   equ     0x0D            ;1st 7-segment display register
disp3   equ     0x0E            ;3rd 7-segment display register
disp2   equ     0x0F            ;2nd 7-segment display register
disp4   equ     0x10            ;4th 7-segment display register
sec     equ     0x11            ;1 second counter
; Bit Definitions .........................................................
#define RP0     status,5
#define T0IF    intcon,2        ;TMR0 overflow indication
#define C       status,0

; PROGRAM START ===========================================================
        org     0
        goto    Main

; SUBROUTINES =============================================================
Tab1    addwf   pcl,f           ;adds W to PC
        retlw   b'11000000'     ;displays 0
        retlw   b'11111001'     ;displays 1
        retlw   b'10100100'     ;displays 2
        retlw   b'10110000'     ;displays 3
        retlw   b'10011001'     ;displays 4
        retlw   b'10010010'     ;displays 5
        retlw   b'10000010'     ;displays 6
        retlw   b'11111000'     ;displays 7
        retlw   b'10000000'     ;displays 8
        retlw   b'10010000'     ;displays 9
; .........................................................................
Tab2    addwf   pcl,f           ;adds W to PC
        retlw   B'11111110'     ;1st 7-segment display anode
        retlw   B'11111011'     ;3rd 7-segment display anode
        retlw   B'11111101'     ;2nd 7-segment display anode
        retlw   B'11110111'     ;4th 7-segment display anode
; .........................................................................
Counter clrf    sec             ;clear register sec
        incf    disp1,f         ;increment Disp1
        movlw   0x0A
        subwf   disp1,w         ;subtract 10 from Disp1 and store to W
        btfss   C               ;if C=1 then Disp1 overflowed 9
        goto    Count_A
        clrf    disp1
        incf    disp2,f         ;increment Disp2
        movlw   0x0A
        subwf   disp2,w         ;subtract 10 from Disp2 and store to W
        btfss   C               ;if C=1 then Disp2 overflowed 9
        goto    Count_A
        clrf    disp2
        incf    disp3,f         ;increment Disp3
        movlw   0x0A
        subwf   disp3,w         ;subtract 10 from Disp3 and store to W
        btfss   C               ;if C=1 then Disp3 overflowed 9
        goto    Count_A
        clrf    disp3
        incf    disp4,f         ;increment Disp4
        movlw   0x0A
        subwf   disp4,w         ;subtract 10 from Disp4 and store to W
        btfss   C               ;if C=1 then Disp4 overflowed 9
        goto    Count_A
        clrf    disp4
Count_A return
; .........................................................................
Display incf    c_disp,f        ;increment multiplexer counter
        movlw   0x04
        subwf   c_disp,w
        btfsc   C               ;if C=1, c_disp overflowed 3
        clrf    c_disp          ;clear c_disp if overflowed
        movlw   0xFF
        movwf   porta           ;display off
        movlw   disp1
        movwf   fsr             ;indirect disp1 address
        movf    c_disp,w        ;copy c_disp to W
        addwf   fsr,f           ;add c_disp to indirect disp1
                                ;address to select one of the
                                ;disp1-4 registers
        movf    indf,w          ;load selected register disp to W
        call    Tab1            ;digit coded to character
        movwf   portb           ;send a character to port B
        movf    c_disp,w
        call    Tab2            ;select the display anode
        movwf   porta           ;switch corresponding transistor
        return

; MAIN PROGRAM ============================================================
Main    bsf     RP0             ;RAM bank 0
        movlw   b'11010011'
        movwf   optreg          ;TMR0 and prescaler configuration
        movlw   0x00
        movwf   portb           ;port B is input
        movlw   0xF0
        movwf   porta           ;bits 0-3 of port A are outputs
        bcf     RP0             ;RAM bank 0
        movlw   0xFF
        movwf   porta           ;all displays off
        clrf    disp1           ;clear register Disp1
        clrf    disp2           ;clear register Disp2
        clrf    disp3           ;clear register Disp3
        clrf    disp4           ;clear register Disp4
        clrf    c_disp          ;clear multiplexer counter
; .........................................................................
Main_A  bcf     T0IF            ; clear T0IF
        clrf    tmr0            ; clear TMR0 and prescaler
Main_B  btfss   T0IF            ;T0IF test
        goto    Main_B
        call    Display         ;display multiplex subroutine
        incf    sec,f           ;increment of the counter of seconds
        movlw   0xC7
        subwf   sec,w           ;subtracts 199 from sec and store to W
        btfsc   C               ;if C=1 then register sec did ;not overflow
        call    Counter         ;if sec overflowed then Counter ;is called
        goto    Main_A          ;jump to the beginning of the ;loop
; =========================================================================
        end